package gui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.Collection;
import java.util.List;

import mst.Edge;
import mst.Point;

public class DrawPanel extends Canvas {

	private static final long serialVersionUID = 1L;
	private boolean cordinatePrintable = false;
	private double rateWidth = 1;
	private double rateHeight = 1;
	private int minX;
	private int minY;
	private int gabX = 20;
	private int gabY = 20;
	private Collection<Point> points = null; //  @jve:decl-index=0:
	private List<Edge> edges = null;
	/**
	 * This is the default constructor
	 */
	Graphics graphics; //  @jve:decl-index=0:	private static Image buffer;

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setMinimumSize(new Dimension(500, 500));
	}

	public void setCordinatePrintable(boolean cordinatePriintable) {
		this.cordinatePrintable = cordinatePriintable;
	}

	public void drawLines(List<Edge> list) {
		edges = list;
		for (Edge edge : list) {
			drawLine(edge);
		}
	}

	public void drawPoints(Collection<Point> points) {
		this.points = points;
		for (Point point : points) {
			drawPoint(point);
		}
	}

	public void drawLine(Edge edge) {
		drawLine(edge.getP1(), edge.getP2());
	}

	public void drawLine(Point p1, Point p2) {
		Graphics g1 = getGraphics();
		g1.setColor(Color.BLACK);
		g1.drawLine((int) ((p1.x - minX) * rateWidth) + gabX, (int) ((p1.y - minY) * rateHeight) + gabY, (int) ((p2.x - minX) * rateWidth)
				+ gabX, (int) ((p2.y - minY) * rateHeight) + gabY);
	}

	public void removeLine(Edge edge) {
		removeLine(edge.getP1(), edge.getP2());
	}

	public void removeLine(Point p1, Point p2) {
		Graphics g1 = getGraphics();
		g1.setColor(Color.BLACK);
		g1.setXORMode(getBackground());
		g1.drawLine((int) ((p1.x - minX) * rateWidth) + gabX, (int) ((p1.y - minY) * rateHeight) + gabY, (int) ((p2.x - minX) * rateWidth)
				+ gabX, (int) ((p2.y - minY) * rateHeight) + gabY);
	}

	public void drawPoint(Point p) {

		int x = (int) ((p.x - minX) * rateWidth) + gabX;
		int y = (int) ((p.y - minY) * rateHeight) + gabY;
		Graphics g1 = getGraphics();
		g1.fillRect(x - 1, y - 1, 3, 3);
		if (cordinatePrintable)
			g1.drawString("(" + p.x + "," + p.y + ")", x + 5, y);
	}

	public void drawPoint(java.awt.Point p) {
		// TODO Auto-generated method stub
		int x = (int) ((p.x - minX) * rateWidth) + gabX;
		int y = (int) ((p.y - minY) * rateHeight) + gabY;
		Graphics g1 = getGraphics();
		g1.fillRect(x - 1, y - 1, 3, 3);
		if (cordinatePrintable)
			g1.drawString(p.x + "," + p.y, x + 5, y);

	}

	public void clearRect() {
		// TODO Auto-generated method stub
		Graphics g1 = getGraphics();
		if (g1 != null)
			g1.clearRect(0, 0, getWidth(), getHeight());

	}

	public DrawPanel() {
		super();
		initialize();
	}

	public void setDefaultRate() {

		rateHeight = 1;
		rateWidth = 1;
	}

	public void setGabs(int gabX, int gabY) {
		this.gabX = gabX;
		this.gabY = gabY;
	}

	public void setRange(int maxX, int minX, int maxY, int minY) {
		this.minX = (int) (minX);
		this.minY = (int) (minY);
		rateHeight = ((double) (getHeight() - gabY * 2) / (double) (maxY - minY));
		rateWidth = ((double) (getWidth() - gabX * 2) / (double) (maxX - minX));
	}

	Dimension dimension = null; //  @jve:decl-index=0:

	public void paint(Graphics g) {
		if (dimension != null) {
			rateHeight = rateHeight * ((double) getSize().height / (double) dimension.height);
			rateWidth = rateWidth * ((double) getSize().width / (double) dimension.width);
			dimension = getSize();
		}
		if (edges != null) {
			drawLines(edges);
		}
		if (points != null) {
			drawPoints(points);
		}

	}
}
