package gui;

import java.awt.*;
import java.io.*;
import java.util.*;

import javax.swing.*;

import mst.*;
import mst.Point;
import java.awt.Font;
import java.awt.Dimension;

public class MinimumSTView extends JFrame {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JButton jButtonStart = null;
	private DrawPanel drawPanel = null;
	private JPanel controlPanel = null;
	private JSlider jSlider = null;
	private JLabel jLabelMaximum = null;
	private JPanel jBottomPanel = null;
	private JButton jButtonLoad = null;
	private JCheckBox jCheckBoxPrintCoordinate = null;
	private JButton jButtonSave = null;
	private JLabel jLabelPointNum = null;
	private InterfaceMST spanningTree = new SpanningTree();
	private int maxCount;
	private boolean isManual = false;
	private Graph graph = null;  //  @jve:decl-index=0:

	private void setFonts() {
		Enumeration<Object> enumeration = UIManager.getDefaults().keys();
		Font f=new Font("",Font.PLAIN,12);
		while (enumeration.hasMoreElements()) {
			Object object=enumeration.nextElement();
			if (object instanceof javax.swing.plaf.FontUIResource) {
				UIManager.put (object, f);
			}
		}
	}

	/**
	 * This is the default constructor
	 */
	public MinimumSTView() {
		super();
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		initialize();
		setVisible(true);
		Collection<Point> temp = new HashSet<Point>();
		temp.add(new Point(1, 1));
		temp.add(new Point(1, 2));
		temp.add(new Point(2, 1));
		// graphics=drawPanel.getGraphics();
		// AbstractConvexHull.setGraphics(graphics);
		getDrawPanel().clearRect();
		clearDrawPanel();
		long maxmem = Runtime.getRuntime().maxMemory() >> 20;
		System.out.println("maxmem=" + maxmem + "mb");
		if (maxmem >= 2048) {
			maxCount = 20000;
		} else if (maxmem >= 1024) {
			maxCount = 15000;
		} else if (maxmem >= 512) {
			maxCount = 10000;
		} else {
			maxCount = 6000;
		}
	}

	/*
	 * public void update(Graphics g) { paint(g); }
	 */
	public void clearDrawPanel() {
		getDrawPanel().clearRect();
	}

	/**
	 * This method initializes jButtonLoad
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonLoad() {
		if (jButtonLoad == null) {
			jButtonLoad = new JButton();
			jButtonLoad.setText("ҷ");
			jButtonLoad.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					Load();
				}
			});
		}
		return jButtonLoad;
	}

	private void Save() {
		FileDialog fd = new FileDialog(this, "", FileDialog.SAVE);
		fd.setVisible(true);
		if (fd.getFile() != null) {
			File f = new File(fd.getDirectory(), fd.getFile());
			try {
				PrintWriter writer = new PrintWriter(f);
				writer.println(spanningTree.getMST().size());
				for (Edge edge : spanningTree.getMST()) {
					writer.println(edge);

				}
				writer.flush();
				writer.close();

			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}

	}

	private void Load() {
		FileDialog fd = new FileDialog(this, "", FileDialog.LOAD);

		fd.setVisible(true);
		int maxX = Integer.MIN_VALUE;
		int maxY = Integer.MIN_VALUE;
		int minX = Integer.MAX_VALUE;
		int minY = Integer.MAX_VALUE;
		if (fd.getFile() != null) {
			File f = new File(fd.getDirectory(), fd.getFile());
			graph = new Graph();

			time = System.currentTimeMillis();
			try {
				Scanner reader = new Scanner(new FileInputStream(f));
				int size = reader.nextInt();

				for (int i = 0; i < size; i++) {
					int x = reader.nextInt();
					int y = reader.nextInt();
					graph.add(new mst.Point(x, y));

					if (x > maxX) {
						maxX = x;
					}
					if (y > maxY) {
						maxY = y;
					}
					if (y < minY) {
						minY = y;
					}
					if (x < minX) {
						minX = x;
					}
				}
				getDrawPanel().setRange(maxX, minX, maxY, minY);
				run(graph);

			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * This method initializes jCheckBoxPrintCoordinate
	 * 
	 * @return javax.swing.JCheckBox
	 */
	private JCheckBox getJCheckBoxPrintCoordinate() {
		if (jCheckBoxPrintCoordinate == null) {
			jCheckBoxPrintCoordinate = new JCheckBox();
			jCheckBoxPrintCoordinate.setText("ǥ ǥ");
		}
		return jCheckBoxPrintCoordinate;
	}

	/**
	 * This method initializes jButtonSave
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonSave() {
		if (jButtonSave == null) {
			jButtonSave = new JButton();
			jButtonSave.setText("ϱ");
			jButtonSave.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					Save();
				}
			});
		}
		return jButtonSave;
	}

	public static void main(String args[]) {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}
		new MinimumSTView();
	}

	/**
	 * This method initializes jBottomPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJBottomPanel() {
		if (jBottomPanel == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.VERTICAL;
			gridBagConstraints.gridy = -1;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.gridx = -1;
			jBottomPanel = new JPanel();
			jBottomPanel.setLayout(new BorderLayout());
			jBottomPanel.add(getControlPanel(), BorderLayout.NORTH);
		}
		return jBottomPanel;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(710, 646);
		this.setResizable(true);
		this.setMinimumSize(new Dimension(800, 646));
		this.setContentPane(getJContentPane());
		this.setTitle("JFrame");
		setFonts();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getDrawPanel(), BorderLayout.CENTER);
			jContentPane.add(getJBottomPanel(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes controlPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getControlPanel() {
		if (controlPanel == null) {

			jLabelPointNum = new JLabel();
			jLabelPointNum.setText(" ");
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(java.awt.FlowLayout.RIGHT);
			controlPanel = new JPanel();
			controlPanel.setLayout(flowLayout);
			controlPanel.add(jLabelPointNum, null);
			controlPanel.add(getJSlider(), null);
			controlPanel.add(getJLabel(), null);
			controlPanel.add(getJCheckBoxPrintCoordinate(), null);
			controlPanel.add(getJButtonLoad(), null);
			controlPanel.add(getJButtonSave(), null);
			controlPanel.add(getJButtonStart(), null);
		}
		return controlPanel;
	}

	/**
	 * This method initializes drawPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private DrawPanel getDrawPanel() {
		if (drawPanel == null) {
			drawPanel = new DrawPanel();
			drawPanel.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					if (!isManual) {
						isManual = true;
						graph = new Graph();
						getDrawPanel().clearRect();
						getDrawPanel().setGabs(0, 0);
					}
					getDrawPanel().drawPoint(e.getPoint());
					graph.add(new Point(e.getPoint().x, e.getPoint().y));
					getJButtonStart().setText(graph.getPoints().size() + ":׸");
					
				}
			});
			drawPanel.setPreferredSize(new Dimension(600, 600));
		}
		return drawPanel;
	}

	/**
	 * This method initializes jButtonStart
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonStart() {
		if (jButtonStart == null) {
			jButtonStart = new JButton();
			jButtonStart.setText("ڵ׸");
			jButtonStart.setPreferredSize(new Dimension(95, 25));
			jButtonStart.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if(graph.size()<3){
						return;			
					}
					if (!isManual) {
						int size = getJSlider().getValue();
						getDrawPanel().setGabs(20, 20);
						graph = new Graph();
						time = System.currentTimeMillis();
						int w = getDrawPanel().getWidth() - 40;
						int h = getDrawPanel().getHeight() - 40;
						Random rand = new Random(System.currentTimeMillis());
						for (int i = 0; i < size; i++) {
							graph.add(new mst.Point(rand.nextInt(w), rand.nextInt(h)));
						}
						getDrawPanel().setDefaultRate();
						run(graph);
					} else {
						time = System.currentTimeMillis();
						isManual = false;
						getDrawPanel().setGabs(0, 0);
						getJButtonStart().setText("ڵ׸");
						run(graph);
					}

				}
			});
		}
		return jButtonStart;
	}

	private long time;

	private void run(Graph graph) {
		System.out.println("ready : " + (System.currentTimeMillis() - time) + " ms");
		getDrawPanel().setCordinatePrintable(getJCheckBoxPrintCoordinate().isSelected());
		getDrawPanel().clearRect();
		Collection<mst.Point> points = graph.getPoints();
		getDrawPanel().drawPoints(points);
		java.util.List<Edge> edges = spanningTree.makeMST(graph);
		getDrawPanel().drawLines(edges);
		System.out.println("complete : " + (System.currentTimeMillis() - time) + " ms");
	}

	private JLabel getJLabel() {
		if (jLabelMaximum == null) {
			jLabelMaximum = new JLabel();
			jLabelMaximum.setText("50");
			jLabelMaximum.setPreferredSize(new Dimension(50, 15));
		}
		return jLabelMaximum;

	}

	/**
	 * This method initializes jSlider
	 * 
	 * @return javax.swing.JSlider
	 */
	private JSlider getJSlider() {
		if (jSlider == null) {
			jSlider = new JSlider();
			long maxmem = Runtime.getRuntime().maxMemory();
			maxCount=(int)Math.sqrt(maxmem/2.0);
			System.out.println("maxmem=" + maxmem + "byte" +", maxcount="+maxCount);
			jSlider.setMaximum(maxCount);
			jSlider.setPreferredSize(new Dimension(300, 23));
			jSlider.setValue(50);
			jSlider.setMinimum(3);
			jSlider.addChangeListener(new javax.swing.event.ChangeListener() {
				public void stateChanged(javax.swing.event.ChangeEvent e) {
					getJLabel().setText(getPointNumber() + "");
				}
			});
		}
		return jSlider;
	}

	public int getPointNumber() {
		return getJSlider().getValue();
	}

	public void paint(Graphics g) {
		super.paint(g);
		getDrawPanel().paint(g);
	}

	@Override
	public void repaint() {
		super.repaint();
		getDrawPanel().repaint();
	}

	@Override
	public void paintAll(Graphics g) {
		super.paintAll(g);
		getDrawPanel().paint(g);
	}

	public void update(Graphics g) {
		super.update(g);
	}

} //  @jve:decl-index=0:visual-constraint="45,-26"
