package mst;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class Graph {
	Set<Point> pointSet;
	Entry[] entries;
	Point[] points;
	MinHeap<Edge> heap;
	private boolean isDisable = false;

	public Graph() {
		// TODO Auto-generated constructor stub

		pointSet = new HashSet<Point>();

	}

	public void add(Point p) {
		if (isDisable) {
			System.out.println("߸ ");
			return;
		}
		pointSet.add(p);
	}

	public void end() {
		points = new Point[pointSet.size()];
		int count = 0;
		for (Point p : pointSet) {
			points[count] = p;
			count++;
		}

		entries = new Entry[points.length - 1];

		points[0].setIndex(0);
		for (int i = 1; i < points.length; i++) {
			points[i].setIndex(i);
			entries[i - 1] = new Entry(i);
		}

		heap = new MinHeap<Edge>(entries.length - 1);

		int cpus = Runtime.getRuntime().availableProcessors();

		//System.out.println(cpus);

		if (cpus > 1 && pointSet.size() > 2000) {

			@SuppressWarnings("unchecked")
			List<Entry> list[] = new List[cpus];
			for (int i = 0; i < cpus; i++) {
				list[i] = new ArrayList<Entry>();
			}

			for (int i = 0; i < entries.length; i++) {

				list[i % cpus].add(entries[i]);

			}
			for (int i = 0; i < cpus; i++) {
				new Thr(list[i]).start();
			}

		} else {
			new Thr(Arrays.asList(entries)).run();
		}
		//System.out.println(graph.size());
		while (heap.size != entries.length) {
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	class Thr extends Thread {
		List<Entry> list;

		public Thr(List<Entry> entries) {
			// TODO Auto-generated constructor stub
			this.list = entries;
		}

		@Override
		public void run() {
			// TODO Auto-generated method stub
			for (Entry entry : list) {
				entry.init();
				entry.sort();
				Edge edge = entry.getEdge();
				heap.insert(edge);
			}
		}
	}

	public Edge getMinEdge() {
		//List<Edge> a=new ArrayList<Edge>();
		// ߿ ּ ġ
		Edge edge = heap.ExtractMin();

		Point p1 = edge.getP1();

		Edge temp = entries[p1.index - 1].getEdge();

		if (temp != null) {
			heap.insert(temp);
		}

		return edge;
	}

	public Collection<Point> getPoints() {
		return pointSet;
	}

	class MinHeap<T extends Comparable<T>> {

		private T[] heap = null;
		private int size = 0;

		@SuppressWarnings("unchecked")
		public MinHeap() {
			heap = (T[]) new Comparable[1024];
		}

		@SuppressWarnings("unchecked")
		public MinHeap(int size) {
			heap = (T[]) new Comparable[size];
		}

		public boolean isEmpty() {
			return size == 0;
		}

		public synchronized void insert(T element) {
			if (size >= heap.length) {
				T[] temp = Arrays.copyOf(heap, heap.length * 2);
				heap = temp;
			}
			heap[size] = element;
			int i = size;
			int p = getParent(i);
			while (p >= 0 && heap[p].compareTo(heap[i]) > 0) {
				T temp = heap[i];
				heap[i] = heap[p];
				heap[p] = temp;
				i = p;
				p = getParent(p);
			}
			size++;
		}

		public T getMinimun() {
			return heap[0];
		}

		public T ExtractMin() {
			T min = heap[0];
			size--;
			heap[0] = heap[size];
			heapify(0);
			return min;
		}

		private void heapify(int i) {
			if (!hasChild(i))
				return;
			int k = getMinimumChild(i);
			if (heap[i].compareTo(heap[k]) > 0) {
				T temp = heap[i];
				heap[i] = heap[k];
				heap[k] = temp;
				heapify(k);
			}
		}

		private boolean hasChild(int i) {
			return (i * 2 + 1) < size;
		}

		private int getParent(int i) {
			return (i - 1) / 2;
		}

		private int getMinimumChild(int i) {
			if ((i * 2 + 2) < size) {
				return heap[i * 2 + 1].compareTo(heap[i * 2 + 2]) < 0 ? i * 2 + 1 : i * 2 + 2;
			} else if (i * 2 + 1 < size) {
				return i * 2 + 1;
			} else {
				return -1;
			}
		}

		public String toString() {
			String str = "";
			for (int i = 0; i < size; i++) {
				str += i + " : " + heap[i] + "\n";
			}
			return str;
		}
	}

	class Entry {
		int start;
		int index;
		int[] ends;

		public Entry(int start) {
			this.start = start;
		}

		public Edge getEdge() {
			// TODO Auto-generated method stub
			if (index < 0) {
				return null;
			}

			Edge edge = new Edge(points[start], points[ends[index]]);
			index--;
			return edge;
		}

		public void init() {
			int size = start;
			ends = new int[size];

			for (int i = 0; i < size; i++) {
				ends[i] = i;
			}

			index = size - 1;
		}

		public void sort() {
			long[] keys = new long[ends.length];
			for (int i = 0; i < ends.length; i++) {
				keys[i] = Point.distance(points[i], points[start]);
			}
			QuickSort.sort(ends, keys);

		}

	}

	public int size() {
		// TODO Auto-generated method stub
		return pointSet.size();
	}

}

class QuickSort {

	static public void sort(int[] list, long[] keys) {
		partition(list, 0, list.length - 1, keys);

	}

	static private void partition(int[] list, int start, int end, long[] keys) {

		if (end <= start)
			return;

		int pivot = list[end];

		int temp = list[start];

		int s = start;

		int e = end;

		for (int i = start; i < end; i++) {

			if (keys[temp] > keys[pivot]) {

				list[s] = temp;

				s++;

				temp = list[s];

			} else {

				list[e] = temp;

				e--;

				temp = list[e];

			}

		}

		list[e] = pivot;

		partition(list, start, e - 1, keys);

		partition(list, e + 1, end, keys);

	}

}
